using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using SharpCharts.Base.Data;
using System.Xml.Serialization;
using System.Windows.Media;
using SharpCharts.Base.ChartObject;

namespace SharpCharts.Base.SharpScript
{
    public class ATR : Indicator
    {
    	public ATR() : this(14)
    	{ }
    
        public ATR(int period) : base()
        {
        	this.Period = period;
            base.PanelCount = 0;
            base.Calculate = Base.SharpScript.Calculate.OnPriceChange;
        }


        protected override void OnStateChange()
        {
            
            switch (base.State)
            {
                case State.Finalize:
                    break;
                case State.Initialize:
                	AddPlot(new Plot("Average True Range", new LinePlotStyle(), Brushes.Blue, 2));
                    break;
                case State.StartUp:
                    break;
                default:
                    break;
            }
        }

        protected override void OnBarUpdate()
        {
            if (CurrentBar == 0)
            {
            	Values[0][0] = High[0] - Low[0];
            }
            else
            {
            	double trueRange = Math.Max(Math.Max(High[0] - Low[0], Math.Abs(High[0] - Close[1])), Math.Abs(Low[0] - Close[1]));
            	Values[0][0] = ((Math.Min(CurrentBar + 1, this.Period) - 1) * Values[0][1] + trueRange) / Math.Min(CurrentBar + 1, this.Period);
            }
        }

        private int period = 14;
        [Input]
        public int Period
        {
            get { return period; }
            set 
            {
                period = Math.Max(1, value);
                NotifyPropertyChanged("Period");
            }
        }

    }
    
    public static partial class AddOrGetIndicator
    {
    	public static ATR ATR(SharpScriptBase owner, int period)
    	{
    		return ATR(owner, owner.Input, period);
    	}
    	
    	public static ATR ATR(SharpScriptBase owner, ISeries<double> input, int period)
    	{
    		return CacheIndicator<ATR>(owner, input, new object[] { period });
    	}
    }
}