using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using SharpCharts.Base.Data;
using System.Xml.Serialization;
using System.Windows.Media;
using SharpCharts.Base.ChartObject;

namespace SharpCharts.Base.SharpScript
{
    public class CMO : Indicator
    {
        SMA smaSd;
        Series<double> sd;


        public CMO()
            : this(14)
        { }

        public CMO(int period)
            : base()
        {
            this.Period = period;
            base.PanelCount = 0;
            base.Calculate = Base.SharpScript.Calculate.OnPriceChange;
        }


        protected override void OnStateChange()
        {

            switch (base.State)
            {
                case State.Finalize:
                    break;
                case State.Initialize:
                    AddPlot(new Plot("CMO", new LinePlotStyle(), Brushes.Green, 2));
                    break;
                case State.StartUp:
                    this.sd = new Series<double>(this);
                    this.smaSd = AddOrGetIndicator.SMA(this, this.sd, this.Period);
                    break;
                default:
                    break;
            }
        }

        protected override void OnBarUpdate()
        {
            if (CurrentBar == 0)
            {
                this.sd[0] = 0.0d;
                return;
            }

            this.sd[0] = Math.Abs(Input[0] - Input[1]);

            double len = Input[0] - Input[Math.Min(CurrentBar, this.Period)];
            if (len == 0.0d)
            {
                Values[0][0] = 0.0d;
            }
            else
            {
                Values[0][0] = 100 * len / (this.smaSd[0] * Math.Min(CurrentBar, this.Period));
            }
        }

        private int period = 14;
        [Input]
        public int Period
        {
            get { return period; }
            set
            {
                period = Math.Max(1, value);
                NotifyPropertyChanged("Period");
            }
        }

    }

    public static partial class AddOrGetIndicator
    {
        public static CMO CMO(SharpScriptBase owner, int period)
        {
            return CMO(owner, owner.Input, period);
        }

        public static CMO CMO(SharpScriptBase owner, ISeries<double> input, int period)
        {
            return CacheIndicator<CMO>(owner, input, new object[] { period });
        }
    }
}