using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using SharpCharts.Base.Data;
using System.Xml.Serialization;
using System.Windows.Media;
using SharpCharts.Base.ChartObject;

namespace SharpCharts.Base.SharpScript
{
    public class HHigh : Indicator
    {
    	double highestHigh = double.MinValue;
    	int highestBar = 0;
    	
    	Series<int> hBar;
    	
        public HHigh() : this(14)
        { }

        public HHigh(int period) : base()
        {   
            this.Period = period;
            base.PanelCount = 1;
            base.Calculate = Base.SharpScript.Calculate.OnPriceChange;
        }


        protected override void OnStateChange()
        {
            
            switch (base.State)
            {
                case State.Finalize:
                    break;
                case State.Initialize:
                
                	AddPlot(new Plot("Highest High", new LinePlotStyle(), new Pen(Brushes.Green, 2)));
                	this.hBar = new Series<int>(this);
                    break;
                case State.StartUp:
                    break;
                default:
                    break;
            }
        }

        protected override void OnBarUpdate()
        {
            if (!Input.HasValue(0))
            	return;
        	
        	if (CurrentBar < this.Period)
        	{
        		if (this.Input[0] >= this.highestHigh)
        		{
	        		this.highestHigh = this.Input[0];
	        		highestBar = CurrentBar;
	        	}
	        	
	        	Values[0][0] = this.highestHigh;
	        	this.hBar[0] = CurrentBar - this.highestBar;
        	}
        	else
        	{
        		if (CurrentBar - highestBar >= this.Period)
        		{
        			this.highestHigh = double.MinValue;

					for (int i = this.Period; i-- > 1; )
        			{
        				
        				if (!Input.HasValue(i))
        					continue;
        					
        				if (Input[i] >= this.highestHigh)
        				{
        					this.highestHigh = Input[i];
        					this.highestBar = CurrentBar - i;
        				}
        			}
        		}
        		
        		
        		if (Input[0] >= this.highestHigh)
        		{
        			this.highestHigh = Input[0];
        			this.highestBar = CurrentBar;
        		}
        			
        		Values[0][0] = this.highestHigh;
        		this.hBar[0] = CurrentBar - this.highestBar;        		
        	}
        }

        private int period = 14;
        [Input]
        public int Period
        {
            get { return period; }
            set 
            {
                period = Math.Max(1, value);
                NotifyPropertyChanged("Period");
            }
        }
        
        [XmlIgnore]
        public ISeries<int> HBar
        {
        	get { return hBar; }
        }

    }
    
    public static partial class AddOrGetIndicator
    {
    	public static HHigh HHigh(SharpScriptBase owner, int period)
    	{
    		return HHigh(owner, owner.Input, period);
    	}
    	
    	public static HHigh HHigh(SharpScriptBase owner, ISeries<double> input, int period)
    	{
    		return CacheIndicator<HHigh>(owner, input, new object[] { period });
    	}
    }
}