using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using SharpCharts.Base.Data;
using System.Xml.Serialization;
using System.Windows.Media;
using SharpCharts.Base.ChartObject;

namespace SharpCharts.Base.SharpScript
{
    public class LLow : Indicator
    {
    
    	double lowestLow = double.MaxValue;
    	int lowestBar = 0;
    	Series<int> lBar;
    
        public LLow() : this(14)
        { }

        public LLow(int period) : base()
        {   
            this.Period = period;
            base.PanelCount = 1;
            base.Calculate = Base.SharpScript.Calculate.OnPriceChange;
        }


        protected override void OnStateChange()
        {
            
            switch (base.State)
            {
                case State.Finalize:
                    break;
                case State.Initialize:
                	
                	AddPlot(new Plot("Lowest Low", new LinePlotStyle(), new Pen(Brushes.Red, 2)));
                	this.lBar = new Series<int>(this);
                    break;
                case State.StartUp:
                    break;
                default:
                    break;
            }
        }

        protected override void OnBarUpdate()
        {
            if (!Input.HasValue(0))
            	return;
            	
            if (CurrentBar < this.Period)
            {
            	if (Input[0] <= this.lowestLow)
            	{
            		this.lowestLow = Input[0];
            		this.lowestBar = CurrentBar;
            	}
            	
            	Values[0][0] = this.lowestLow;
            	this.lBar[0] = CurrentBar - this.lowestBar;
            }
            else
            {
            	if (CurrentBar - lowestBar >= this.Period)
            	{
            		this.lowestLow = double.MaxValue;
            		
            		for (int i = this.Period; i-- > 1; )
            		{
            			if (!Input.HasValue(i))
            				continue;
            				
            			if (Input[i] <= lowestLow)
            			{
            				this.lowestLow = Input[i];
            				this.lowestBar = CurrentBar - i;
            			}
            		}
            	}
            	
            	if (Input[0] <= this.lowestLow)
            	{
            		this.lowestLow = Input[0];
            		this.lowestBar = CurrentBar;
            	}
            	
            	Values[0][0] = lowestLow;
            	this.lBar[0] = CurrentBar - this.lowestBar;
            }
        }

        private int period = 14;
        [Input]
        public int Period
        {
            get { return period; }
            set 
            {
                period = Math.Max(1, value);
                NotifyPropertyChanged("Period");
            }
        }
        
        [XmlIgnore]
        public ISeries<int> LBar
        {
        	get { return lBar; }
        }

    }
    
    public static partial class AddOrGetIndicator
    {
    	public static LLow LLow(SharpScriptBase owner, int period)
    	{
    		return LLow(owner, owner.Input, period);
    	}
    	
    	public static LLow LLow(SharpScriptBase owner, ISeries<double> input, int period)
    	{
    		return CacheIndicator<LLow>(owner, input, new object[] { period });
    	}
    }
    
}