using System;
using System.Collections.Generic;

using System.Windows.Media;
using System.IO;
using System.ComponentModel;
using SharpCharts.Base.Data;
using System.Xml.Serialization;

namespace SharpCharts.Base.SharpScript
{
    /// <summary>
    /// Class definition for SMA indicator
    /// </summary>
    public class SMA : Indicator
    {
        
        /// <summary>
        /// Constructor for SMA
        /// </summary>
        public SMA()
            : this(14)
        { }

        public SMA(int period) : base()
        {
            this.period = period;

            base.Calculate = SharpScript.Calculate.OnPriceChange;
        }

        /// <summary>
        /// Updates when the State property changes
        /// </summary>
        protected override void OnStateChange()
        {
            
            switch (base.State)
            {
                case State.Finalize:
                    break;
                case State.Initialize:

                    this.PanelCount = 1;
                    base.AddPlot(new Plot("SMA", new LinePlotStyle(), new Pen(Brushes.Firebrick, 2)));
                    
                    break;
                case State.StartUp:
                    break;
                default:
                    break;
            }
        }
        
        /// <summary>
        /// Updates on incoming tick
        /// </summary>
        protected override void OnBarUpdate()
        {
            if (this.CurrentBar == 0)
            {
                Values[0][0] = Input[0];
            }
            else
            {
                double last = Values[0][1] * Math.Min(this.CurrentBar, Period);
                if (this.CurrentBar >= Period)
                {
                    Values[0][0] = (last + Input[0] - Input[Period]) / Math.Min(this.CurrentBar, Period);
                }
                else
                {
                    Values[0][0] = (last + Input[0]) / (Math.Min(this.CurrentBar, Period) + 1);
                }
            }
        }
        
        
        private int period = 14;
        [Input]
        public int Period
        {
            get { return period; }
            set 
            {
                period = Math.Max(1, value);
                NotifyPropertyChanged("Period");
            }
        }

               
        public override Common.TemplateType TemplateType
        {
            get
            {
                return Common.TemplateType.Default;
            }
        }

                
    }
    
    
    public static partial class AddOrGetIndicator
    {
    	public static SMA SMA(SharpScriptBase owner, int period)
    	{
    		return SMA(owner, owner.Close, period);
    	}
    	
    	public static SMA SMA(SharpScriptBase owner, ISeries<double> input, int period)
    	{
    		return AddOrGetIndicator.CacheIndicator<SMA>(owner, input, new object[] { period });
    	}
    }
}
