using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using SharpCharts.Base.Data;
using System.Xml.Serialization;
using System.Windows.Media;
using SharpCharts.Base.ChartObject;

namespace SharpCharts.Base.SharpScript
{
	/// <summary>	
	/// Triple Exponential Moving Average
	/// </summery>
    public class TEMA : Indicator
    {
    	EMA ema1 = null;
    	EMA ema2 = null;
    	EMA ema3 = null;
    
        public TEMA() : this(14)
        { }

        public TEMA(int period) : base()
        {   
            this.Period = period;
            base.PanelCount = 1;
            base.Calculate = Base.SharpScript.Calculate.OnPriceChange;
        }


        protected override void OnStateChange()
        {
            switch (base.State)
            {
                case State.Finalize:
                    break;
                case State.Initialize:
                	AddPlot(new Plot("TEMA", new LinePlotStyle(), Brushes.Green, 2));
                    break;
                case State.StartUp:
                	this.ema1 = AddOrGetIndicator.EMA(this, this.Period);
                	this.ema2 = AddOrGetIndicator.EMA(this, this.ema1, this.Period);
                	this.ema3 = AddOrGetIndicator.EMA(this, this.ema2, this.Period);
                    break;
                default:
                    break;
            }
        }

        protected override void OnBarUpdate()
        {
            Values[0][0] = 3 * this.ema1[0] - 3 * this.ema2[0] + this.ema3[0];
        }

        private int period = 14;
        [Input]
        public int Period
        {
            get { return period; }
            set 
            {
                period = Math.Max(1, value);
                NotifyPropertyChanged("Period");
            }
        }

    }
    
    public static partial class AddOrGetIndicator
     {
    	public static TEMA TEMA(SharpScriptBase owner, int period)
    	{
    		return TEMA(owner, owner.Input, period);
    	}
    	
    	public static TEMA TEMA(SharpScriptBase owner, ISeries<double> input, int period)
    	{
    		return CacheIndicator<TEMA>(owner, input, new object[] { period });
    	}
    }
}