using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using SharpCharts.Base.Data;
using System.Xml.Serialization;
using System.Windows.Media;
using SharpCharts.Base.ChartObject;

namespace SharpCharts.Base.SharpScript
{
	/// <summary>
	/// Triangular Moving Average
	/// </summery>
    public class TMA : Indicator
    {
    
    	SUM sum = null;
    	
        public TMA() : this(14)
        { }

        public TMA(int period) : base()
        {   
            this.Period = period;
            base.PanelCount = 1;
            base.Calculate = Base.SharpScript.Calculate.OnPriceChange;
        }


        protected override void OnStateChange()
        {
            
            switch (base.State)
            {
                case State.Finalize:
                    break;
                case State.Initialize:
                	AddPlot(new Plot("TMA", new LinePlotStyle(), Brushes.Green, 2));
                    break;
                case State.StartUp:
                	
                	SMA sma = AddOrGetIndicator.SMA(this, this.Input, this.Period);
                	this.sum = AddOrGetIndicator.SUM(this, sma, this.Period);
                
                    break;
                default:
                    break;
            }
        }

        protected override void OnBarUpdate()
        {
            Values[0][0] = this.sum[0] / Math.Min(CurrentBar + 1, this.Period);        	
        }

        private int period = 14;
        [Input]
        public int Period
        {
            get { return period; }
            set 
            {
                period = Math.Max(1, value);
                NotifyPropertyChanged("Period");
            }
        }

    }
    
    public static partial class AddOrGetIndicator
    {
    	public static TMA TMA(SharpScriptBase owner, int period)
    	{
    		return TMA(owner, owner.Close, period);
    	}
    	
    	public static TMA TMA(SharpScriptBase owner, ISeries<double> input, int period)
    	{
    		return AddOrGetIndicator.CacheIndicator<TMA>(owner, input, new object[] { period });
    	}
    }
    
}