using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using SharpCharts.Base.Data;
using System.Xml.Serialization;
using System.Windows.Media;
using SharpCharts.Base.ChartObject;

namespace SharpCharts.Base.SharpScript
{
    public class VIDYA : Indicator
    {
    
    	CMO cmo = null;
    	
        public VIDYA() : this(9, 9)
        { }

        public VIDYA(int period, int volatility) : base()
        {   
            this.Period = period;
            this.Volatility = volatility;
            base.PanelCount = 1;
            base.Calculate = Base.SharpScript.Calculate.OnPriceChange;
        }


        protected override void OnStateChange()
        {
            
            switch (base.State)
            {
                case State.Finalize:
                    break;
                case State.Initialize:
                	AddPlot(new Plot("VMA", new LinePlotStyle(), Brushes.Green, 2));
                    break;
                case State.StartUp:
                	this.cmo = AddOrGetIndicator.CMO(this, this.Volatility);
                    break;
                default:
                    break;
            }
        }

        protected override void OnBarUpdate()
        {
            if (CurrentBar == 0)
            {
            	Values[0][0] = Input[0];
            	return;
            }
            
            double vi = Math.Abs(this.cmo[0]) / 100;
            Values[0][0] = 2 / (double)(this.Period + 1) * vi * (Input[0] - Values[0][1]) + Values[0][1];
        }

        
        
        private int volatility = 9;
        [Input]
        public int Volatility
        {
        	get { return volatility; }
        	set
        	{
        		volatility = Math.Max(1, value);
        		NotifyPropertyChanged("Volatility");
        	}
        }
        
        private int period = 9;
        [Input]
        public int Period
        {
            get { return period; }
            set 
            {
                period = Math.Max(1, value);
                NotifyPropertyChanged("Period");
            }
        }

    }
    
    public static partial class AddOrGetIndicator
    {
    	public static VIDYA VIDYA(SharpScriptBase owner, int period, int volatility)
    	{
    		return VIDYA(owner, owner.Input, period, volatility);
    	}
    	
    	public static VIDYA VIDYA(SharpScriptBase owner, ISeries<double> input, int period, int volatility)
    	{
    		return CacheIndicator<VIDYA>(owner, input, new object[] { period, volatility });
    	}
    }
}