using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using SharpCharts.Base.Data;
using System.Xml.Serialization;
using System.Windows.Media;
using SharpCharts.Base.ChartObject;

namespace SharpCharts.Base.SharpScript
{
    public class WMA : Indicator
    {
    	double priorSum = 0.0d;
    	
        public WMA() : this(14)
        { }

        public WMA(int period) : base()
        {   
            this.Period = period;
            base.PanelCount = 1;
            base.Calculate = Base.SharpScript.Calculate.OnPriceChange;
        }

        protected override void OnStateChange()
        {
            switch (base.State)
            {
                case State.Finalize:
                    break;
                case State.Initialize:
                	AddPlot(new Plot("WMA", new LinePlotStyle(), Brushes.Green, 2));
                    break;
                case State.StartUp:
                    break;
                default:
                    break;
            }
        }

		
        protected override void OnBarUpdate()
        {        	
        
        	if (CurrentBar >= Period)
        	{
        		if (this.IsFirstTickOfBar)
        		{
        			this.priorSum = this.priorSum + Input[1] - (CurrentBar == Period ? 0.0d : Input[Period + 1]);
        		}
        		int multiplier = Period * (Period + 1) / 2;
        		double prWeightedPrice = Values[0][1] * multiplier - priorSum + Input[0] * Period;
				Values[0][0] = prWeightedPrice / multiplier;
        	}
        	else
        	{
        		if (CurrentBar == 0)
        		{
        			this.priorSum = 0.0d;
        			Values[0][0] = Input[0];
        			return;
        		}
        		
        		if (this.IsFirstTickOfBar)
        		{
        			this.priorSum = this.priorSum + Input[1];
        		}
        			
        		int prMultiplier = CurrentBar * (CurrentBar + 1) / 2;
        		double prWeightedPrice = Values[0][1] * prMultiplier + Input[0] * (CurrentBar + 1);
        		int multiplier = (CurrentBar + 1) * (CurrentBar + 2) / 2;
				Values[0][0] = prWeightedPrice / multiplier;
			}		
        }
        
        
        private int period = 14;
        [Input]
        public int Period
        {
            get { return period; }
            set 
            {
                period = Math.Max(1, value);
                NotifyPropertyChanged("Period");
            }
        }

    }
    
    public static partial class AddOrGetIndicator
    {
    	public static WMA WMA(SharpScriptBase owner, int period)
    	{
    		return WMA(owner, owner.Close, period);
    	}
    	
    	public static WMA WMA(SharpScriptBase owner, ISeries<double> input, int period)
    	{
    		return AddOrGetIndicator.CacheIndicator<WMA>(owner, input, new object[] { period });
    	}
    }
}