using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using SharpCharts.Base.Data;
using System.Xml.Serialization;
using System.Windows.Media;
using SharpCharts.Base.ChartObject;

namespace SharpCharts.Base.SharpScript
{
    public class HilbertTransform : Indicator
    {
    	Series<double> Smooth;
		Series<double> Detrender;
		Series<double> I1;
		Series<double> Q1;		
		Series<double> jI;
		Series<double> jQ;		
		Series<double> I2;
		Series<double> Q2;
		Series<double> Re;
		Series<double> Im;				
		
    	Series<double> cyclePeriod = null;
    	Series<double> cycleSmoothPeriod = null;
    	
    	
    
        public HilbertTransform() : this(10)
        { }

        public HilbertTransform(int wmaPeriod) : base()
        {   
            this.Period = wmaPeriod;
            base.PanelCount = 0;
            base.Calculate = Base.SharpScript.Calculate.OnPriceChange;
        }


        protected override void OnStateChange()
        {
            
            switch (base.State)
            {
                case State.Finalize:
                    break;
                case State.Initialize:
                	
                	AddPlot(new Plot("In Phase", new LinePlotStyle(), Brushes.Green, 2));
                	AddPlot(new Plot("Quadrature", new LinePlotStyle(), Brushes.Orange, 2));
                	
                	AddLine(new Line("Zero Line"));
                
                    break;
                case State.StartUp:
                
                	this.Smooth = new Series<double>(this);
                	this.Detrender = new Series<double>(this);
                	this.I1 = new Series<double>(this);
                	this.Q1 = new Series<double>(this);
                	this.jI = new Series<double>(this);
                	this.jQ = new Series<double>(this);
                	this.I2 = new Series<double>(this);
                	this.Q2 = new Series<double>(this);
                	this.Re = new Series<double>(this);
                	this.Im = new Series<double>(this);
                	this.cyclePeriod = new Series<double>(this);
                	this.cycleSmoothPeriod = new Series<double>(this);
                
                    break;
                default:
                    break;
            }
        }

        protected override void OnBarUpdate()
        {
            if( this.CurrentBar < 50)
				return;
						
			Smooth[0] = (4 * Median[0] + 3 * Median[1] + 2 * Median[2] + Median[3]) / 10;
			Detrender[0] = (0.0962 * Smooth[0] + 0.5769 * Smooth[2] - 0.5769 * Smooth[4] - 0.0962 * Smooth[6]) * (0.075 * cyclePeriod[1] + .54);
			
			//InPhase and Quadrature components			
			Q1[0] = (0.0962 * Detrender[0] + 0.5769 * Detrender[2] - 0.5769 * Detrender[4] - 0.0962 * Detrender[6]) * (0.075 * cyclePeriod[1] + 0.54);			
			I1[0] = Detrender[3];
			
			//Advance the phase of I1 and Q1 by 90 degrees
			jI[0] = (0.0962 * I1[0] + 0.5769 * I1[2] - 0.5769 * I1[4] - 0.0962 * I1[6]) * (0.075 * cyclePeriod[1] + .54);
			jQ[0] = (0.0962 * Q1[0] + 0.5769 * Q1[2] - 0.5769 * Q1[4] - 0.0962 * Q1[6]) * (0.075 * cyclePeriod[1] + .54);
			
			//Phasor Addition
			I2[0] = I1[0] - jQ[0];
			Q2[0] = Q1[0] + jI[0];
			
			//Smooth the I and Q components before applying the discriminator
			I2[0] = 0.2 * I2[0] + 0.8 * I2[1];
			Q2[0] = 0.2 * Q2[0] + 0.8 * Q2[1];
			
			//Homodyne Discriminator
			Re[0] = I2[0] * I2[1] + Q2[0] * Q2[1];
			Im[0] = I2[0] * Q2[1] - Q2[0] * I2[1];
			Re[0] = 0.2 * Re[0] + 0.8 * Re[1];
			Im[0] = 0.2 * Im[0] + 0.8 * Im[1];
			
			double rad2Deg = 180.0 / (4.0 * Math.Atan (1));
			
			if(Im[0] != 0 && Re[0] != 0)
			{
				cyclePeriod[0] = 360 / (Math.Atan(Im[0] / Re[0]) * rad2Deg);
			}
			
			if(cyclePeriod[0] > (1.5 * cyclePeriod[1]))
			{
				cyclePeriod[0] = 1.5 * cyclePeriod[1];
			}
			
			if(cyclePeriod[0] < (0.67 * cyclePeriod[1]))
			{
				cyclePeriod[0] = 0.67 * cyclePeriod[1];
			}
			
			if(cyclePeriod[0] < 6)
			{
				cyclePeriod[0] = 6;
			}
			
			if(cyclePeriod[0] > 50)
			{
				cyclePeriod[0] = 50;
			}
			
			cyclePeriod[0] = 0.2 * cyclePeriod[0] + 0.8 * cyclePeriod[1];
			cycleSmoothPeriod[0] = 0.33 * cyclePeriod[0] + 0.67 * cycleSmoothPeriod[1];
			
			
			Values[0][0] = I1[0];
			Values[1][0] = Q1[0];	
        }
        
        [XmlIgnore]
        public ISeries<double> InPhase
        {
        	get { return Values[0]; }
        }
        
        [XmlIgnore]
        public ISeries<double> Quadrature
        {
        	get { return Values[1]; }
        }
        
        [XmlIgnore]
        public ISeries<double> CycleSmoothPeriod
        {
        	get { return cycleSmoothPeriod; }
        }
        
        [XmlIgnore]
        public ISeries<double> CyclePeriod
        {
        	get { return cyclePeriod; }
        }

        private int wmaPeriod = 10;
        [Input]
        [System.ComponentModel.DisplayName("WMA Period")]
        public int Period
        {
            get { return wmaPeriod; }
            set 
            {
                wmaPeriod = Math.Max(1, value);
                NotifyPropertyChanged("Period");
            }
        }
    }
    
    public static partial class AddOrGetIndicator
    {
    	public static HilbertTransform HilbertTransform(SharpScriptBase owner, int wmaPeriod)
    	{
    		return HilbertTransform(owner, owner.Input, wmaPeriod);
    	}
    	
    	public static HilbertTransform HilbertTransform(SharpScriptBase owner, ISeries<double> input, int wmaPeriod)
    	{
    		return CacheIndicator<HilbertTransform>(owner, input, new object[] { wmaPeriod });
    	}
    }
    
}