using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using SharpCharts.Base.Common;
using SharpCharts.Base.Data;
using System.Xml.Serialization;
using System.Windows.Media;
using SharpCharts.Base.ChartObject;

namespace SharpCharts.Base.SharpScript
{
    public class TSSuperTrend : Indicator
    {
    
    	ISeries<double> movingAvg = null;
    	Series<bool> trend = null;
    	
    	double th = 0.0d;
    	double tl = double.MaxValue;
    	
    	int thisBar = -1;
    	
    	double offset;
    	ATR atr = null;
    	HomodyneDiscriminator homodyneDiscriminator = null;
    	    
    	HHigh hHigh = null;
    	HHigh cHigh = null;
    	LLow cLow = null;
    	LLow lLow = null;
    	
        public TSSuperTrend() : this(14, SuperTrendMAType.EMA, 2.618, 14, SuperTrendMode.ATR)
        { }

        public TSSuperTrend(int period, SuperTrendMAType movingAverageType, double multiplier, int smooth, SuperTrendMode superTrendMode) : base()
        {   
        	this.SuperTrendMode = superTrendMode;
        	this.Multiplier = multiplier;
        	this.Smooth = smooth;
        	this.MovingAverageType = movingAverageType;
            this.Period = period;
            base.PanelCount = 1;
            base.Calculate = Base.SharpScript.Calculate.OnPriceChange;
        }


        protected override void OnStateChange()
        {
            
            switch (base.State)
            {
                case State.Finalize:
                    break;
                case State.Initialize:
                	AddPlot(new Plot("Up Trend", new HashPlotStyle(), Brushes.Blue, 3));
                	AddPlot(new Plot("Down Trend", new HashPlotStyle(), Brushes.Red, 3));
                    break;
                case State.StartUp:
                
                	this.trend = new Series<bool>(this);
                	
                	switch (this.MovingAverageType)
                	{
                		case SuperTrendMAType.HMA:
                			this.movingAvg = AddOrGetIndicator.HMA(this, Input, this.Smooth);
                			break;
                		case SuperTrendMAType.SMA:
                			this.movingAvg = AddOrGetIndicator.SMA(this, Input, this.Smooth);
                			break;
                		case SuperTrendMAType.SMMA:
                			this.movingAvg = AddOrGetIndicator.SMMA(this, Input, this.Smooth);
                			break;
                		case SuperTrendMAType.TEMA:
                			this.movingAvg = AddOrGetIndicator.TEMA(this, Input, this.Smooth);
                			break;
                		case SuperTrendMAType.TMA:
                			this.movingAvg = AddOrGetIndicator.TMA(this, Input, this.Smooth);
                			break;
                		case SuperTrendMAType.VIDYA:
                			this.movingAvg = AddOrGetIndicator.VIDYA(this, Input, this.Smooth, this.Smooth);
                			break;
                		case SuperTrendMAType.VWMA:
                			this.movingAvg = AddOrGetIndicator.VWMA(this, Input, this.Smooth);
                			break;
                		case SuperTrendMAType.WMA:
                			this.movingAvg = AddOrGetIndicator.WMA(this, Input, this.Smooth);
                			break;
                		case SuperTrendMAType.EMA:
                		default:
                			this.movingAvg = AddOrGetIndicator.EMA(this, Input, this.Smooth);
                			break;
                	}
                	
                	switch (this.SuperTrendMode)
            		{
            			case SuperTrendMode.ATR:
            				this.atr = AddOrGetIndicator.ATR(this, this.Period);
		            		break;
		            	case SuperTrendMode.Adaptive:
	            			this.homodyneDiscriminator = AddOrGetIndicator.HomodyneDiscriminator(this, Input);
	            			this.atr = AddOrGetIndicator.ATR(this, this.Period);
		            		break;
		            	default:
	            			this.hHigh = AddOrGetIndicator.HHigh(this, High, this.Period);
	            			this.cHigh = AddOrGetIndicator.HHigh(this, Close, this.Period);
	            			this.cLow = AddOrGetIndicator.LLow(this, Close, this.Period);
	            			this.lLow = AddOrGetIndicator.LLow(this, Low, this.Period);
		            		break;
		            }
                	
                    break;
                default:
                    break;
            }
        }

        protected override void OnBarUpdate()
        {
        
            switch (this.SuperTrendMode)
            {
            	case SuperTrendMode.ATR:
            		this.offset = this.atr[0] * this.Multiplier;
            		break;
            	case SuperTrendMode.Adaptive:
            		this.offset = this.atr[0] * this.homodyneDiscriminator[0] / 10;
            		break;
            	default:
            		this.offset = this.Multiplier * Math.Max(this.hHigh[0] - this.cLow[0], this.cHigh[0] - this.lLow[0]);
            		break;
            }
            
            if (CurrentBar < 1)
        	{
        		//this.movingAvg[0] = Input[0];
        		this.trend[0] = true;
        		Values[0][0] = Values[1][0] = Input[0];
        		return;
        	}
            
            /*
            if (this.IsFirstTickOfBar)
            {
                _prevColor = _tempColor;
			}
			*/
			
            this.trend[0] = Close[0] > (Values[1].HasValue(1) ? Values[1][1] : Input[1]) ? true : Close[0] < (Values[0].HasValue(1) ? Values[0][1] : Input[1]) ? false : this.trend[1];

            if (this.trend[0] && !this.trend[1])
            {
                this.th = High[0];
                
                Values[0][0] = Math.Max(this.movingAvg[0] - this.offset, this.tl);
                Values[1].Reset();
                
                if (ShowArrow)
                {
                	InvokeAsync(() => Draw.ArrowUp(this, CurrentBar.ToString(), Time[0], Values[0][0] - TickSize()));
                }
                
                if(PlayAlert && this.thisBar != CurrentBar)
                {
                    this.thisBar = CurrentBar;
                    this.Alert(this.thisBar.ToString(), "SuperTrend given long signal");
                }
                
                if (this.PaintBars)
                {
                	this.BarBrush[0] = this.UpBarBrush;
                }
                
            }
            else
            {
                if (!this.trend[0] && this.trend[1])
                {
                    this.tl = Low[0];
                    Values[1][0] = Math.Min(this.movingAvg[0] + this.offset, this.th);
                    Values[0].Reset();
                    
                    if (ShowArrow)
                    {
                    	InvokeAsync(() => Draw.ArrowDown(this, CurrentBar.ToString(), Time[0], Values[1][0] + TickSize()));
                    }
                    
                    if (PlayAlert && this.thisBar != CurrentBar)
                    {
                        this.thisBar = CurrentBar;
                        Alert(this.thisBar.ToString(), "SuperTrend given a short signal");
                    }
                    
                    if (this.PaintBars)
                    {
                    	this.BarBrush[0] = this.DownBarBrush;
                    }
                    
                }
                else
                {
                    if (this.trend[0])
                    {
                        Values[0][0] = (this.movingAvg[0] - this.offset) > Values[0][1] ? (this.movingAvg[0] - this.offset) : Values[0][1];
                        this.th = Math.Max(this.th, High[0]);
                        Values[1].Reset();
                    }
                    else
                    {
                        Values[1][0] = (this.movingAvg[0] + this.offset) < Values[1][1] ? (this.movingAvg[0] + this.offset) : Values[1][1];
                        this.tl = Math.Min(this.tl, Low[0]);
                        Values[0].Reset();
                    }
                    
                    if (this.trend[0] == this.trend[1])
                    {
                    	BarBrush[0] = null;
	                    RemoveChartObject(CurrentBar.ToString());
	                }
                    
                }
            }
            

			
            if (!this.PaintBars) 
                return;
			
			
			/*
            CandleOutlineColor = _tempColor;

            BarColor = Open[0] < Close[0] && ChartControl.ChartStyleType == ChartStyleType.CandleStick
                           ? Color.Transparent
                           : _tempColor;
            */            
            
        }
        
        [XmlIgnore]
        public SuperTrendMode[] SuperTrendModes
        {
        	get { return (SuperTrendMode[])Enum.GetValues(typeof(SuperTrendMode)); }
        }
        
        private SuperTrendMode superTrendMode = SuperTrendMode.ATR;
        [Input]
        public SuperTrendMode SuperTrendMode
        {
        	get { return superTrendMode; }
        	set 
        	{
	        	superTrendMode = value; 
	        	NotifyPropertyChanged("SuperTrendMode");
	        }
        }

        private int period = 14;
        [Input]
        public int Period
        {
            get { return period; }
            set 
            {
                period = Math.Max(1, value);
                NotifyPropertyChanged("Period");
            }
        }
        
        private double multiplier = 2.618d;
        [Input]
        public double Multiplier
        {
        	get { return multiplier; }
        	set
        	{
        		multiplier = value;
        		NotifyPropertyChanged("Multiplier");
        	}
        }
        
        
        [XmlIgnore]
        public SuperTrendMAType[] SuperTrendMATypes
        {
        	get { return (SuperTrendMAType[])Enum.GetValues(typeof(SuperTrendMAType)); }
        }
        
        private SuperTrendMAType movingAverageType = SuperTrendMAType.EMA;
		[Input]
		public SuperTrendMAType MovingAverageType
		{
			get { return movingAverageType; }
			set
			{
				movingAverageType = value;
				NotifyPropertyChanged("MovingAverageType");
			}
		}
		
		private int smooth = 14;
		[Input]
		public int Smooth
		{
			get { return smooth; }
			set
			{
				smooth = Math.Max(1, value);
				NotifyPropertyChanged("Smooth");
			}
		}
		
		public bool ShowArrow { get; set; }
		
		public bool PaintBars { get; set; }
		
		public bool PlayAlert { get; set; }
		
		private Brush upBarBrush = Brushes.Blue;
		[XmlElement(ElementName="UpBarBrush", Type= typeof(XamlSerializer<Brush>))]
		public Brush UpBarBrush
		{
			get { return upBarBrush; }
			set
			{
				upBarBrush = value;
				if (upBarBrush != null && upBarBrush.CanFreeze)
				{
					upBarBrush.Freeze();
				}
			}
		}
		
		private Brush downBarBrush = Brushes.Pink;
		[XmlElement(ElementName="DownBarBrush", Type= typeof(XamlSerializer<Brush>))]
		public Brush DownBarBrush
		{
			get { return downBarBrush; }
			set
			{
				downBarBrush = value;
				if (downBarBrush != null && downBarBrush.CanFreeze)
				{
					downBarBrush.Freeze();
				}
			}
		}
		
		[XmlIgnore]
		public ISeries<double> UpTrend
		{
			get { return Values[0]; }
		}
		
		[XmlIgnore]
		public ISeries<double> DownTrend
		{
			get { return Values[1]; }
		}
		
    }
    
    public enum SuperTrendMode
    {
    	ATR,
    	DualThrust,
    	Adaptive
    }
    
    public enum SuperTrendMAType
    {
    	EMA,
    	HMA,
    	SMA,
    	SMMA,
    	TEMA,
    	TMA,
    	WMA,
    	VIDYA,
    	VWMA,
    }
    
    public static partial class AddOrGetIndicator
    {
    	public static TSSuperTrend TSSuperTrend(SharpScriptBase owner, int period, SuperTrendMAType movingAverageType, double multiplier, int smooth, SuperTrendMode superTrendMode)
    	{
    		return TSSuperTrend(owner, owner.Input, period, movingAverageType, multiplier, smooth, superTrendMode);
    	}
    	
    	public static TSSuperTrend TSSuperTrend(SharpScriptBase owner, ISeries<double> input, int period, SuperTrendMAType movingAverageType, double multiplier, int smooth, SuperTrendMode superTrendMode)
    	{
    		return CacheIndicator<TSSuperTrend>(owner, input, new object[] { period, movingAverageType, multiplier, smooth, superTrendMode });
    	}
    }
    
}


