using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using SharpCharts.Base.Data;
using System.Xml.Serialization;
using System.Windows.Media;

namespace SharpCharts.Base.SharpScript
{
    public sealed class VWMA : Indicator
    {
        public VWMA()
            : this(14)
        { }
    
    
        public VWMA(int period)
        {
             this.period = period;
            base.Calculate = Base.SharpScript.Calculate.OnPriceChange;
        }


        protected override void OnStateChange()
        {
            
            switch (base.State)
            {
                case State.Finalize:
                    break;
                case State.Initialize:
                    base.PanelCount = 1;
                    base.AddPlot(new Plot("VWMA",new LinePlotStyle(), new Pen(Brushes.Firebrick,2)));
                    break;
                case State.StartUp:
                    break;
                default:
                    break;
            }
        }

        protected override void OnBarUpdate()
        {
            int  numBars = Math.Min(CurrentBar, period);

			double volPriceSum  = 0;
			double volSum		= 0;

			for (int i = 0; i < numBars; i++)
			{
				volPriceSum	+= Input[i] * Volume[i];
				volSum		+= Volume[i];
			}
			
			if (volSum <= double.Epsilon)
				Values[0][0] = Input[0];
			else
				Values[0][0] = volPriceSum / volSum;
        }

        private int period = 14;
        [Input]
        public int Period
        {
            get { return period; }
            set 
            {
                period = Math.Max(1, value);
                NotifyPropertyChanged("Period");
            }
        }

    }
    
    public static partial class AddOrGetIndicator
     {
    	public static VWMA VWMA(SharpScriptBase owner, int period)
    	{
    		return VWMA(owner, owner.Input, period);
    	}
    	
    	public static VWMA VWMA(SharpScriptBase owner, ISeries<double> input, int period)
    	{
    		return CacheIndicator<VWMA>(owner, input, new object[] { period });
    	}
    }
    
}